@@ -135,7 +135,7 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
 
         cmd = PersonCreate('levin',
             role = personRole,
-            fullname='Levin',
+            fullname=u'Levin \xf3',
             email='levin@%s' % domainName
         )
         cmd.execute()
@@ -145,7 +145,7 @@ class InitialiseDomainModel(InitialiseDomainModelBase):
         
         cmd = PersonCreate('natasha',
             role = personRole,
-            fullname='Natasha',
+            fullname=u'Natasha \xf3',
             email='natasha@%s' % domainName
         )
         cmd.execute()
