@@ -1,6 +1,6 @@
-from dm.command.base import * 
-import dm.re
 import re
+import dm.regexps
+from dm.command.base import * 
 
 class PluginCommand(Command):
     "Abstract Plugin command."
@@ -44,7 +44,7 @@ class PluginCommand(Command):
 class PluginCreate(DomainObjectCreate):
     "Command to create a new plugin."
 
-    reservedNames = re.compile('^%s$' % dm.re.reservedPluginName)
+    reservedNames = re.compile('^%s$' % dm.regexps.reservedPluginName)
 
     def __init__(self, name='', **kwds):
         super(PluginCreate, self).__init__(
