@@ -26,18 +26,15 @@ similar aspects to both MPMs and ORMs, this module does not build or execute
 SQL statements and so cannot be classified as an ORM. Instead, this module
 depends on an ORM to move values from the persistence model to the DBRMS.
 
-This module has been tested with the recent stable SQLObject versions, which
-are understood by the author to be 0.7.10, 0.8.7, 0.9.5, and 0.10.0. It has
-also been tested with the python-sqlobject version of Debian Etch (0.7.1).
-
 """
 
+# Todo: Expand for all database systems (not just 'postgres').
 from dm.exceptions import *
 
 import sqlobject
 sqoVer = None
 isSqoSupported = False
-sqoVers = ['0.7', '0.8', '0.9', '0.10']
+sqoVers = ['0.7', '0.8', '0.9', '0.10', '0.11', '0.12']
 if sqlobject.__doc__ != None:
     sqoDocSplit = sqlobject.__doc__.split(' ')
     if len(sqoDocSplit) > 1:
@@ -67,6 +64,7 @@ import sqlobject.main
 from dm.ioc import *
 from dm.dictionarywords import *
 import mx.DateTime
+import weakref
 
 debug = RequiredFeature('Debug')
 logger = RequiredFeature('Logger')
@@ -86,16 +84,19 @@ class ConnectionFacade(object):
         dbName = self.dictionary[DB_NAME]
         if self.dictionary.has_key(DB_URI):
             uri = self.dictionary[DB_URI]
+        elif dbType == 'sqlite':
+            uri = "sqlite://%s" % dbName
         else:
             uri = "%s://%s:%s@%s/%s" % (dbType, dbUser, dbPass, dbHost, dbName)
+        logger.info('Database: Connecting to %s.' % uri)
         try:
             self.connection = connectionForURI(uri)
             self.connection.makeConnection()
         except Exception, inst:
-            msg = 'Could not connect to database: %s' % (inst)
+            msg = 'Database: Could not make a connection: %s' % (inst)
             logger.critical(msg)
-            raise Exception, msg
-        logger.info('Connected to database OK.')
+            raise Exception, "Could not make a database connection (see system log for connection URI: %s)." % self.dictionary[LOG_PATH]
+        # Todo: Check connection.dbEncoding == 'UTF8'.
     
     def getConnection(self):
         return self.connection
@@ -127,8 +128,12 @@ class DatabaseFacade(object):
         def findRecord(self, className, *args, **kwds):
             "Retreive record of domain object."
             record = None
-            if moddebug and debug:
-                logger.debug('Finding %s record in database.' % (className))
+            if moddebug:
+                logger.info('Database: Finding %s record in database.' % (className))
+            import os
+            if 'SHIT' in os.environ:
+                if className == 'Role':
+                    raise Exception, "What's the caller?"
             if self.isSelectById(kwds) and len(kwds) == 1:
                 objectId = kwds['id']
                 try:
@@ -192,6 +197,8 @@ class DatabaseFacade(object):
 
         def findRecords(self, className, *args, **kwds):
             "Retreive records of domain objects."
+            if moddebug:
+                logger.info('Database: Finding %s records in database: %s %s' % (className, args, kwds))
             recordClass = self.getRecordClass(className)
             if self.isSelectById(kwds):
                 records = recordClass.selectByKeywordsWithId(**kwds)
@@ -236,7 +243,11 @@ class DatabaseFacade(object):
         def createRecord(self, className, *args, **kwds):
             "Create record of domain object."
             recordClass = self.getRecordClass(className)
-            recordClass.assertCreateParams(*args, **kwds)
+            if not self.connection.dictionary[DB_MIGRATION_IN_PROGRESS]:
+                recordClass.assertCreateParams(*args, **kwds)
+            if debug:
+                message = "Creating new %s record with %s" % (className, kwds)
+                logger.debug(message)
             try:
                 newRecord = recordClass(*args, **kwds)
             except Exception, inst:
@@ -248,7 +259,7 @@ class DatabaseFacade(object):
        
         def getRecordClass(self, className):
             "Returns record class for domain object type name."
-            if not className: 
+            if not className:
                 raise Exception, "Can't get record class without class name."
             try:
                 mapperClass = self.mappers[className]
@@ -288,7 +299,7 @@ class DatabaseFacade(object):
             except:
                 try:
                     mapperClass.createTable()
-                except Exception, inst: 
+                except Exception, inst:
                     msg = "Couldn't create table for class '%s': " % className 
                     msg += "Error: %s. " % inst
                     attributes = str(mapperClass.meta.dom.attributeNames)
@@ -486,7 +497,7 @@ class MetaMapper(MetaBase):
         self.isUnique = self.dom.isUnique
         self.isCached = self.dom.isCached
         dbName = self.makeSafeTableName(self.dom.dbName or self.dom.name)
-        if sqoVer in ['0.8', '0.9', '0.10']:
+        if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12']:
             self.domName = dbName
             self.dbName = dbName
         if sqoVer == '0.7':
@@ -589,12 +600,12 @@ class MetaMapperAttribute(MetaBase):
         self.isValueRef = self.dom.isValueRef
         self.isAssociateList = self.dom.isAssociateList
         self.isDomainObjectRef = self.dom.isDomainObjectRef
-        if sqoVer in ['0.8', '0.9', '0.10'] and self.isDomainObjectRef:
+        if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12'] and self.isDomainObjectRef:
             self.typeName = self.makeSafeFieldName(self.dom.typeName)
         else:
             self.typeName = self.dom.typeName
         self.dbName = self.makeSafeFieldName(self.dom.dbName or self.dom.name)
-        if self.dom.isValueRef and hasattr(self.dom, 'default'): 
+        if self.dom.isValueRef and hasattr(self.dom, 'default'):
             self.default = self.dom.default
         elif self.isDomainObjectRef:
             if hasattr(self.dom, 'default') and not self.dom.default:
@@ -679,7 +690,7 @@ class MetaMapperAttribute(MetaBase):
         elif issubclass(self.dom.__class__, dm.dom.meta.ImageFile):
             return None
         elif issubclass(self.dom.__class__, dm.dom.meta.DomainObjectRef):
-            if sqoVer in ['0.8', '0.9', '0.10']:
+            if sqoVer in ['0.8', '0.9', '0.10', '0.11', '0.12']:
                 dbName = self.dbName
             elif sqoVer in ['0.7']:
                 dbName = self.dbName + 'ID'
@@ -709,7 +720,7 @@ class Mapper(SQLObject):
         super(Mapper, self).__init__(**kwds)
 
     def isUnique(self):
-        if 'isUnique' in self.map: 
+        if 'isUnique' in self.map:
             return self.map['isUnique']
         else:
             return False
@@ -717,7 +728,7 @@ class Mapper(SQLObject):
     isUnique = classmethod(isUnique)
     
     def isCached(self):
-        if 'isCached' in self.map: 
+        if 'isCached' in self.map:
             isCached = self.map['isCached']
         else:
             isCached = False
@@ -747,39 +758,44 @@ class Mapper(SQLObject):
     
     def getDomainObject(self, loadedList=None):
         "Method to 'Lazy Load' mapped domain object."
+        if moddebug:
+            msg = "Database: Getting %s #%s domain object from record." % (self.getClassName(), self.id)
+            logger.info(msg)
         if loadedList == None:
-            loadedList = dict()
-        if not self.domainObject:
-            obj = self.createDomainObject() 
-            self.domainObject = obj 
-            self.domainObject.record = self
+            loadedList = set()
+        #logger.info("Loaded list: %s" % loadedList)
+        if not self.domainObject or not self.domainObject():
+            if moddebug:
+                msg = "Database: - no object, so recreating domain object."
+                logger.info(msg)
+            #print "Domain object (1): %s" % self.domainObject
+            domainObject = self.getDomainClass()()
+            domainObject.record = self
+            self.domainObject = weakref.ref(domainObject)
+            #print "Domain object (2): %s" % self.domainObject
             self.loadDomainObject(loadedList, loadMapper=False)
         else:
+            if moddebug:
+                msg = "Database: - yes object, but checking whether to reload values."
+                logger.info(msg)
             if not self.isCached():
-                if not self.domainObject in loadedList:
+                if not self.domainObject() in loadedList:
+                    if moddebug:
+                        msg = "Database: --- not in 'just loaded' list, so reloading values."
+                        logger.info(msg)
                     self.loadDomainObject(loadedList)
                 else:
-                    if moddebug and debug:
-                        message = "Avoided circular loading of %s #%s." % (
-                            self.getClassName(), self.id
-                        )
-                        logger.debug(message)
+                    if moddebug:
+                        msg = "Database: --- in 'just loaded' list, so skipping reload."
+                        logger.info(msg)
             else: 
-                if moddebug and debug:
-                    message = "Reusing %s #%s values." % (
-                        self.getClassName(), self.id
-                    )
-                    logger.debug(message)
-        return self.domainObject
+                if moddebug:
+                    msg = "Database: --- cached type, so skipping reload."
+                    logger.info(msg)
+        return self.domainObject()
 
     def createDomainObject(self):
         "Reinstantiates pre-recorded domain object."
-        domainClass = self.getDomainClass()
-        if moddebug and debug:
-            message = "Reinstantiating %s from record." % self.getClassName()
-            logger.debug(message)
-        domainObject = domainClass()
-        return domainObject
        
     def getClassName(self):
         return self.__class__.__name__
@@ -795,61 +811,67 @@ class Mapper(SQLObject):
         return registry.getDomainClass(self.getDomainClassName())
 
     def loadDomainObject(self, loadedList, loadMapper=True):
-        loadedList[self.domainObject] = self.domainObject
+        loadedList.add(self.domainObject())
         if loadMapper:
             self.loadMapperAttributesFromRecord()
         self.loadDomainObjectFromMapperAttributes(loadedList)
 
     def loadMapperAttributesFromRecord(self):
         "Maps record values to mapper attributes."
-        if moddebug and debug:
-            message = "Reloading mapper attributes from record."
-            logger.debug(message)
+        if moddebug:
+            message = "Database: BOOM! Pulling %s #%s attributes from database." % (self.__class__.__name__, self.id)
+            logger.info(message)
         self.sync()
+        if moddebug:
+            logger.info("Database: DONE")
 
     def loadDomainObjectFromMapperAttributes(self, loadedList):
-        if moddebug and debug:
-            message = "Loading %s from mapper attributes..." % (
-                self.getClassName()
+        if moddebug:
+            msg = "Database: - loading %s #%s from mapper attributes..." % (
+                self.getClassName(),
+                self.id
             )
-            logger.debug(message)
+            logger.info(msg)
         currentVersion = None
-        if self.domainObject.meta.isTemporal:
-            currentVersion = self.domainObject.temporalHistory.getCurrent()
-        self.domainObject.id = self.id
+        if not self.domainObject():
+            raise Exception("Missing domain object reference on record.")
+        #print "Domain object (3): %s" % self.domainObject
+        if self.domainObject().meta.isTemporal:
+            currentVersion = self.domainObject().temporalHistory.getCurrent()
+        self.domainObject().id = self.id
         for metaAttr in self.meta.attributes:
             dbName = metaAttr.dbName
             domName = metaAttr.domName
-            if moddebug and debug:
-                message = "Loading %s %s attribute." % (
+            if False and moddebug:
+                msg = "Database: - loading %s %s attribute." % (
                     self.getClassName(), domName,
                 )
-                logger.debug(message)
+                logger.info(msg)
             if domName == 'parent' and metaAttr.isDomainObjectRef:
                 # Continue if it's already loaded....
-                if self.parent and self.parent.domainObject:
-                    mapperValue = self.parent.domainObject
-                    self.domainObject.parent = mapperValue
+                if self.parent and self.parent.domainObject():
+                    mapperValue = self.parent.domainObject()
+                    self.domainObject().parent = mapperValue
                     continue
             if metaAttr.dom.isTemporal:
                 # We don't persist temporal attributes on the parent.
                 #     - we need to look in the temporal model.
-                if self.domainObject.meta.isTemporal:
+                if self.domainObject().meta.isTemporal:
                     if currentVersion:
                         mappedValue = getattr(currentVersion, domName)
                     else:
                         mappedValue = metaAttr.dom.createInitialValue(
-                            self.domainObject
+                            self.domainObject()
                         )
                 else:
                     # Temporal property histories are virtual ATM.
-                    p = metaAttr.dom.createTemporalCollection(self.domainObject)
+                    p = metaAttr.dom.createTemporalCollection(self.domainObject())
                     currentPropertyVersion = p.findFirstDomainObject(loadedList)
                     if currentPropertyVersion:
                         mappedValue = currentPropertyVersion.recordedValue
                     else:
                         mappedValue = metaAttr.dom.createInitialValue(
-                            self.domainObject
+                            self.domainObject()
                         )
             elif metaAttr.isDomainObjectRef:
                 mapper = getattr(self, dbName)
@@ -874,31 +896,31 @@ class Mapper(SQLObject):
                 mappedValue = getattr(self, dbName)
             else:
                 mappedValue = getattr(self, dbName)
-            setattr(self.domainObject, domName, mappedValue)
-
-        if moddebug and debug:
-            message = "Loaded %s from record #%s" % (
-                self.getClassName(), self.id,
+            setattr(self.domainObject(), domName, mappedValue)
+        if moddebug:
+            msg = "Database: Done loading %s #%s from mapper attributes." % (
+                self.getClassName(),
+                self.id
             )
-            logger.debug(message)
+            logger.info(msg)
 
     def saveDomainObject(self):
         "Sets attributes of record object from domain object."
         isChanged = False
         isChangedTemporal = False
         currentVersion = None
-        if self.domainObject.meta.isTemporal:
-            currentVersion = self.domainObject.temporalHistory.getCurrent()
+        if self.domainObject().meta.isTemporal:
+            currentVersion = self.domainObject().temporalHistory.getCurrent()
             if not currentVersion:
                 isChangedTemporal = True
 
             if not isChangedTemporal:
-                for metaAttr in self.domainObject.meta.attributes:
+                for metaAttr in self.domainObject().meta.attributes:
                     if metaAttr.isTemporal:
                         if metaAttr.isAssociateList:
                             recordedRegister = getattr(currentVersion, metaAttr.name)
                             recordedList = recordedRegister.getObjectList()
-                            actualRegister = getattr(self.domainObject, metaAttr.name)
+                            actualRegister = getattr(self.domainObject(), metaAttr.name)
                             actualList = actualRegister.getObjectList()
                             recordedLen = len(recordedList)
                             actualLen = len(actualList)
@@ -921,22 +943,22 @@ class Mapper(SQLObject):
                         else:
                             domName = metaAttr.name
                             recordedValue = getattr(currentVersion, domName)
-                            actualValue = getattr(self.domainObject, domName)
+                            actualValue = getattr(self.domainObject(), domName)
                             if actualValue != recordedValue:
                                 isChangedTemporal = True
                                 break
             if isChangedTemporal:
                 #print "Is changed!!!!!!! Creating new current version of associate list."
-                loadedList = {self.domainObject: self.domainObject}
-                history = self.domainObject.temporalHistory
+                loadedList = set([self.domainObject()])
+                history = self.domainObject().temporalHistory
                 currentVersion = history.create(loadedList=loadedList)
 
-                for metaAttr in self.domainObject.meta.attributes:
+                for metaAttr in self.domainObject().meta.attributes:
                     if metaAttr.isTemporal and metaAttr.isAssociateList:
                         isListChanged = False
                         recordedRegister = getattr(currentVersion, metaAttr.name)
                         recordedList = recordedRegister.getObjectList()
-                        actualRegister = getattr(self.domainObject, metaAttr.name)
+                        actualRegister = getattr(self.domainObject(), metaAttr.name)
                         actualList = actualRegister.getObjectList()
                         recordedLen = len(recordedList)
                         actualLen = len(actualList)
@@ -973,17 +995,17 @@ class Mapper(SQLObject):
                                 #print "Recorded register now: %s" % [i for i in recordedRegister]
 
         for metaAttr in self.meta.attributes:
-            domValue = getattr(self.domainObject, metaAttr.domName)
+            domValue = getattr(self.domainObject(), metaAttr.domName)
             # Check value with most recent record.
             if metaAttr.dom.isTemporal:
                 # We don't persist temporal attributes on the parent.
                 #     - we need to look in the temporal model.
-                if self.domainObject.meta.isTemporal:
+                if self.domainObject().meta.isTemporal:
                     setattr(currentVersion, metaAttr.domName, domValue)
                 else:
                     # Temporal property histories are virtual ATM.
-                    p = metaAttr.dom.createTemporalCollection(self.domainObject)
-                    loadedList = {self.domainObject: self.domainObject}
+                    p = metaAttr.dom.createTemporalCollection(self.domainObject())
+                    loadedList = set([self.domainObject()])
                     current = p.findFirstDomainObject(loadedList=loadedList)
                     if current == None or current.recordedValue != domValue:
                         p.create(recordedValue=domValue, loadedList=loadedList)
@@ -1006,11 +1028,9 @@ class Mapper(SQLObject):
         if isChangedTemporal:
             currentVersion.save()
         if isChanged:
-            if moddebug and debug:
-                message = "Updating RDBMS with %s mapper value." % (
-                    self.meta.domName
-                )
-                logger.debug(message)
+            if moddebug:
+                message = "Database: BOOM! Putting %s #%s attributes to database." % (self.__class__.__name__, self.id)
+                logger.info(message)
             try:
                 # Call SQLObject.
                 self.syncUpdate()
@@ -1020,6 +1040,8 @@ class Mapper(SQLObject):
                 )
                 logger.error(message)
                 raise
+            if moddebug:
+                logger.info("Database: DONE")
 
     def coerceKwds(self, kwds):
         "Converting keyword names to database names."
@@ -1028,12 +1050,12 @@ class Mapper(SQLObject):
                 if metaAttr.isDomainObjectRef:
                     mapper = kwds[metaAttr.domName]
                     del kwds[metaAttr.domName]
-                    if sqoVer in ['0.10']:
+                    if sqoVer in ['0.10', '0.11', '0.12']:
                         dbName = metaAttr.dbName
                     elif sqoVer in ['0.7', '0.8', '0.9']:
                         dbName = metaAttr.dbName + 'ID'
                     if mapper:
-                        if sqoVer in ['0.10']:
+                        if sqoVer in ['0.10', '0.11', '0.12']:
                             kwds[dbName] = mapper
                         elif sqoVer in ['0.7', '0.8', '0.9']:
                             kwds[dbName] = mapper.id
@@ -1046,18 +1068,25 @@ class Mapper(SQLObject):
 
     coerceKwds = classmethod(coerceKwds)
 
-    def getAttributeDbName(self, domName):
-        return self.meta.getAttribute(domName).dbName
+    def selectByKeywords(self, **kwds):
+        "Selects records from database using domain object attribute names."
+        self.coerceKwds(kwds)
+        if moddebug:
+            msg = "Database: BOOM!! Call to selectBy() SQL kwds: %s" % kwds
+            logger.info(msg)
+        try:
+            selection = self.selectBy(**kwds)
+        except Exception, inst:
+            msg = "Couldn't execute selectBy() on '%s' with %s: %s" % (
+                self.meta.domName, kwds, inst
+            )
+            raise KforgeDbError, msg
+        if moddebug:
+            logger.info("Database: DONE")
+        return selection
+       
+    selectByKeywords = classmethod(selectByKeywords)
 
-    getAttributeDbName = classmethod(getAttributeDbName)
-
-    def makeSqlName(self, name):
-        if name[-2:] == 'ID':
-            name = name[:-2] + '_id'
-        return name
-
-    makeSqlName = classmethod(makeSqlName)
-    
     def startsWith(self, value, attributeName, **kwds):
         sqlSafeValue = sqlrepr(value, 'postgres')[1:-1] # drop quotes
         dbName = self.getAttributeDbName(attributeName)
@@ -1067,7 +1096,7 @@ class Mapper(SQLObject):
         self.coerceKwds(kwds)
         for name in kwds:
             value = kwds[name]
-            if sqoVer in ['0.10']:
+            if sqoVer in ['0.10', '0.11', '0.12']:
                 if self.meta.attributeDbNames[name].isDomainObjectRef:
                     name += '_id'
             # todo: expand for all database systems
@@ -1078,13 +1107,8 @@ class Mapper(SQLObject):
             else:
                 sqlEquals = "%s = %s" % (sqlSafeName, sqlSafeValue)
             sqlWhere += " AND ( " + sqlEquals + " ) "
-        try:
-            sqlWhere = "(" + sqlWhere + ")"
-            selection = self.select(sqlWhere)
-        except Exception, inst:
-            msg = "Couldn't execute select() with %s: %s" % (sqlWhere, inst)
-            raise KforgeDbError, msg
-        return selection
+        sqlWhere = "(" + sqlWhere + ")"
+        return self.selectWhere(sqlWhere)
         
     startsWith = classmethod(startsWith)
 
@@ -1095,7 +1119,8 @@ class Mapper(SQLObject):
             dbNames.append(dbName)
         sqlLikeList = []
         # todo: expand for all database systems
-        sqlSafeQuery = sqlrepr(userQuery, 'postgres')[1:-1] # drop quotes
+        sqlSafeQuery = sqlrepr(userQuery, 'postgres')
+        sqlSafeQuery = sqlSafeQuery[1:-1] # Drop quotes (added later).
         if spaceSplit:
             sqlSafeTerms = sqlSafeQuery.split(' ')
         else:
@@ -1117,10 +1142,9 @@ class Mapper(SQLObject):
         sqlEqualsList = []
         for name in kwds:
             value = kwds[name]
-            if sqoVer in ['0.10']:
+            if sqoVer in ['0.10', '0.11', '0.12']:
                 if self.meta.attributeDbNames[name].isDomainObjectRef:
                     name += '_id'
-            # todo: expand for all database systems
             sqlSafeValue = sqlrepr(value, 'postgres')
             sqlSafeName = self.makeSqlName(name)
             if sqlSafeValue.upper() == 'NULL':
@@ -1133,12 +1157,7 @@ class Mapper(SQLObject):
             sqlWhere = '((%s) AND (%s))' % (sqlLike, sqlEquals)
         else:
             sqlWhere = sqlLike
-        try:
-            selection = self.select(sqlWhere)
-        except Exception, inst:
-            msg = "Couldn't execute select() with %s: %s" % (sqlWhere, inst)
-            raise KforgeDbError, msg
-        return selection
+        return self.selectWhere(sqlWhere)
        
     search = classmethod(search)
 
@@ -1147,7 +1166,7 @@ class Mapper(SQLObject):
         sqlEqualsList = []
         for name in kwds:
             value = kwds[name]
-            if sqoVer in ['0.10'] and name != 'id':
+            if sqoVer in ['0.10', '0.11', '0.12'] and name != 'id':
                 if self.meta.attributeDbNames[name].isDomainObjectRef:
                     name += '_id'
             if name == 'id':
@@ -1167,12 +1186,7 @@ class Mapper(SQLObject):
             sqlEqualsList.append(sqlEquals)
         sqlEquals = "(%s)" % ") AND (".join(sqlEqualsList)
         sqlWhere = '(%s)' % (sqlEquals)
-        try:
-            selection = self.select(sqlWhere)
-        except Exception, inst:
-            msg = "Couldn't execute select() with %s: %s" % (sqlWhere, inst)
-            raise KforgeDbError, msg
-        return selection
+        return self.selectWhere(sqlWhere)
 
     selectByKeywordsWithId = classmethod(selectByKeywordsWithId)
     
@@ -1250,7 +1264,7 @@ class Mapper(SQLObject):
                 )
                 sqlEqualsList.append(sqlEquals)
             else:
-                if sqoVer in ['0.10']:
+                if sqoVer in ['0.10', '0.11', '0.12']:
                     if self.meta.attributeDbNames[name].isDomainObjectRef:
                         name += '_id'
                 sqlSafeName = self.makeSqlName(name)
@@ -1261,28 +1275,38 @@ class Mapper(SQLObject):
                 sqlEqualsList.append(sqlEquals)
         sqlEquals = "(%s)" % ") AND (".join(sqlEqualsList)
         sqlWhere = '(%s)' % (sqlEquals)
+        return self.selectWhere(sqlWhere)
+
+    selectByKeywordsWithTimeInterval = classmethod(
+        selectByKeywordsWithTimeInterval)
+
+    def getAttributeDbName(self, domName):
+        return self.meta.getAttribute(domName).dbName
+
+    getAttributeDbName = classmethod(getAttributeDbName)
+
+    def makeSqlName(self, name):
+        if name[-2:] == 'ID':
+            name = name[:-2] + '_id'
+        return name
+
+    makeSqlName = classmethod(makeSqlName)
+    
+    def selectWhere(self, sqlWhere):
+        # Todo: Write test to check special character are handled OK.
+        if type(sqlWhere) == unicode:
+            sqlWhere = sqlWhere.encode('utf8')
+        if moddebug:
+            msg = "Database: BOOM!! Call to select() SQL WHERE: %s" % sqlWhere
+            logger.info(msg)
         try:
             selection = self.select(sqlWhere)
         except Exception, inst:
             msg = "Couldn't execute select() with %s: %s" % (sqlWhere, inst)
             raise KforgeDbError, msg
+        if moddebug:
+            logger.info("Database: DONE")
         return selection
 
-    selectByKeywordsWithTimeInterval = classmethod(
-        selectByKeywordsWithTimeInterval
-    )
+    selectWhere = classmethod(selectWhere)
     
-    def selectByKeywords(self, **kwds):
-        "Selects records from database using domain object attribute names."
-        self.coerceKwds(kwds)
-        try:
-            selection = self.selectBy(**kwds)
-        except Exception, inst:
-            msg = "Couldn't execute selectBy() on '%s' with %s: %s" % (
-                self.meta.domName, kwds, inst
-            )
-            raise KforgeDbError, msg
-        return selection
-       
-    selectByKeywords = classmethod(selectByKeywords)
-
