@@ -5,7 +5,7 @@ dictionary = RequiredFeature('SystemDictionary')
 
 #
 ## Settings commonly defined in the system dictionary.
-DEBUG = dictionary[dictionarywords.SYSTEM_MODE] != 'production',
+DEBUG = not RequiredFeature('SystemMode').isProduction()
 TEMPLATE_DIRS = (dictionary[dictionarywords.DJANGO_TEMPLATES_DIR],)
 TIME_ZONE = dictionary[dictionarywords.TIMEZONE]
 SECRET_KEY = dictionary[dictionarywords.DJANGO_SECRET_KEY]
