@@ -13,13 +13,10 @@ class TestSettings(unittest.TestCase):
     def test_main(self):
         # stable actual values, check for correct value
         self.failUnlessEqual(settings.TIME_ZONE, 'Europe/Paris')
-        self.failUnlessEqual(
-            settings.SECRET_KEY,
-            '(*YZF@{ZFHW$Rli(EFH(*SEHF(&9S(P&SYGDF(&G9Ged9f7gs'
-        )
         
         # unstable actual values, check for any value
-        self.failUnless(settings.TEMPLATE_DIRS, "Template dirs not set.")
+        self.failUnless(settings.SECRET_KEY)
+        self.failUnless(settings.TEMPLATE_DIRS)
 
         # abstract settings, check for null value
         self.failUnlessEqual(settings.ROOT_URLCONF, '')
