@@ -13,7 +13,6 @@ from dm.datetimeconvertor import RNSDateTimeConvertor
 from dm.datetimeconvertor import DateConvertor
 from dm.datetimeconvertor import RDateConvertor
 from dm.datetimeconvertor import DateOfBirthConvertor
-import Image
 from dm.dictionarywords import IMAGES_DIR_PATH
 from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
 import os
@@ -167,18 +166,18 @@ class MetaDomainAttr(BaseDomainMeta):
         self.typeName = typeName
         self.name = name
         self.dbName = dbName or name
-        if default != NotDefined: 
+        if default != NotDefined:
             self.default = default
         self.title = title        # presentable name
         #if not self.title:
         #    self.title = self.spaceFromCamel(self.name)
         #self.title = self.spaceFromCamel(self.name)
         self.comment = comment
-        self.isEditable = isEditable
-        self.isHidden = isHidden
+        self.isEditable = isEditable # Present it, but don't allow edits.
+        self.isHidden = isHidden     # Don't present it.
         if self.isHidden:
             self.isEditable = False
-        self.isRequired = isRequired
+        self.isRequired = isRequired # Require value.
         self.getChoices = getChoices
         self.isIndexed = isIndexed
         self.isTemporal = isTemporal
@@ -221,17 +220,19 @@ class MetaDomainAttr(BaseDomainMeta):
             valueRepr = self.createValueRepr(domainObject)
         except Exception, inst:
             valueRepr = "!!VALUE_REPR_ERROR: %s!!" % inst
-        return "%s=\'%s\'" % (self.name, valueRepr)
+        return "%s=%s" % (self.name, repr(valueRepr))
 
     def createSortableRepr(self, domainObject):
         return self.createValueRepr(domainObject)
 
+    def createLabelValue(self, domainObject):
+        return unicode(getattr(domainObject, self.name))
+    # Todo: Replace createValueRepr() with createLabelValue() everywhere.
     def createValueRepr(self, domainObject):
-        value = getattr(domainObject, self.name)
-        return unicode(value)
+        return self.createLabelValue(domainObject)
 
     def createLabelRepr(self, domainObject):
-        return self.createValueRepr(domainObject)
+        return self.createValueRepr(domainObject).encode('utf-8')
 
     def createAssociatedNamedValues(self, domainObject):
         raise Exception('No createAssociatedNamedValues method on %s' % self)
@@ -345,11 +346,8 @@ from markdown import Markdown
 class MarkdownText(Text):
     "Models a 'markdown' textual attribute."
 
-    # Todo: Figure out how to render separately from read value?
-    #     Ans: Implement this method:
-    #def createLabelRepr(self, domainObject):
-
     def createLabelRepr(self, domainObject):
+        # Render for reading.
         attrValue = getattr(domainObject, self.name)
         if attrValue:
             md = Markdown(safe_mode='escape')
@@ -372,19 +370,24 @@ class Password(String):
         super(Password, self).__init__(isRequired=isRequired, **kwds)
 
     def createValueRepr(self, domainObject):
+        # Don't represent the password digest...
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            # ...unless data migration is in progress, when it's required.
             return super(Password, self).createValueRepr(domainObject)
         else:
-            return ""
+            return u""
 
     def makeValueFromMultiValueDict(self, multiValueDict):
         passwordText = multiValueDict[self.name]
+        # Make a digest of the password...
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            # ...unless data migration is in progress, it's already digested.
             attrValue = passwordText
         elif passwordText:
             attrValue = self.makeDigest(passwordText)
         else:
-            attrValue = ''
+            # Disable digest matching, nothing digests to '' right?
+            attrValue = ''  
         if moddebug and self.debug:
             msg = "Made %s value: %s" % (self.name, attrValue)
             self.logger.debug(msg)
@@ -472,10 +475,8 @@ class Boolean(ValueObjectAttr):
         boolVal = multiValueDict[self.name]
         if boolVal.__class__ == False.__class__:
             attrValue = boolVal
-        elif boolVal.__class__ == ''.__class__:
-            attrValue = boolVal.lower() in self.trueStrings
         else:
-            attrValue = False
+            attrValue = str(boolVal).lower() in self.trueStrings
         if moddebug and self.debug:
             msg = "Made %s value: %s" % (self.name, attrValue)
             self.logger.debug(msg)
@@ -484,9 +485,9 @@ class Boolean(ValueObjectAttr):
     def createValueRepr(self, domainObject):
         attrValue = getattr(domainObject, self.name)
         if attrValue:
-            return 'on'
+            return u'on'
         else:
-            return ''
+            return u''
 
     def createLabelRepr(self, domainObject):
         attrValue = getattr(domainObject, self.name)
@@ -522,12 +523,11 @@ class Integer(ValueObjectAttr):
         return attrValue
 
     def createValueRepr(self, domainObject):
-        value = getattr(domainObject, self.name)
-        return value
+        return getattr(domainObject, self.name)
 
     def createLabelRepr(self, domainObject):
         value = getattr(domainObject, self.name)
-        return unicode(value)
+        return str(value)
 
 
 class BLOB(ValueObjectAttr):
@@ -572,6 +572,7 @@ class ImageFile(MetaDomainAttr):
                 file.write(content)
                 file.close()
             if self.maxWidth != None:
+                import Image  # Not ideal: avoids dependency when not needed.
                 image = Image.open(filePath)
                 (width, height) = image.size
                 if width > self.maxWidth:
@@ -621,7 +622,7 @@ class DomainObjectAssociation(MetaDomainAttr):
 
     def getDomainRegister(self):
         domainClass = self.getDomainClass()
-        return domainClass.createRegister()
+        return domainClass.principalRegister or domainClass.createRegister()
 
     def getDomainClass(self):
         return self.registry.getDomainClass(self.typeName)
@@ -629,7 +630,7 @@ class DomainObjectAssociation(MetaDomainAttr):
     def getObjectByKey(self, key):
         domainRegister = self.getDomainRegister()
         if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
-            #if hasattr(domainRegister, 'getAll'):
+            # Need all objects when migrating.
             if domainRegister.isStateful:
                 domainRegister = domainRegister.getAll()
         return domainRegister[key]
@@ -674,9 +675,9 @@ class DomainObjectRef(DomainObjectAssociation):
         if value == None:
             return ""
         elif value:
-            return unicode(value.getLabelValue())
+            return value.getLabelValue()
         else:
-            return unicode(value)
+            return value
        
     def makeValueFromMultiValueDict(self, multiValueDict):
         refValue = multiValueDict[self.name]
@@ -867,9 +868,9 @@ class AssociateList(DomainObjectAssociation):
         if register == None:
             return valuesList
         if self.isKeyDomainObject():
-            valuesList = [i.getRegisterKeyValue() for i in register.keys()]
+            valuesList = [unicode(i.getRegisterKeyValue()) for i in register.keys()]
         else:
-            valuesList = [i for i in register.keys()]
+            valuesList = [unicode(i) for i in register.keys()]
         return valuesList
 
     def createLabelRepr(self, domainObject):
@@ -932,7 +933,12 @@ class AssociateList(DomainObjectAssociation):
         return associatedNamedValues
 
     def setAttributeFromMultiValueDict(self, domainObject, multiValueDict):
-        associatedObjectKeys = multiValueDict.getlist(self.name)
+        if hasattr(multiValueDict, 'getlist'):
+            # It's a MultiValueDict (Django 0.96).
+            associatedObjectKeys = multiValueDict.getlist(self.name)
+        else:
+            # It's just a dict (Django 1.0).
+            associatedObjectKeys = multiValueDict[self.name]
         associatedObjectRegister = self.getAssociatedObjectRegister(
             domainObject)
         associationObjectRegister = self.getAssociationObjectRegister(
