@@ -11,12 +11,15 @@ def suite():
     suites = [
         unittest.makeSuite(TestString),
         unittest.makeSuite(TestText),
+        unittest.makeSuite(TestMarkdownText),
         unittest.makeSuite(TestUrl),
         unittest.makeSuite(TestPassword),
         unittest.makeSuite(TestDateTime),
         unittest.makeSuite(TestDate),
+        unittest.makeSuite(TestRDate),
         unittest.makeSuite(TestBoolean_True),
         unittest.makeSuite(TestBoolean_False),
+        unittest.makeSuite(TestInteger_Zero),
         unittest.makeSuite(TestInteger_NonZero),
         unittest.makeSuite(TestInteger_Negative),
         unittest.makeSuite(TestImageFile),
@@ -101,6 +104,17 @@ class TestText(MetaAttrTestCase):
     dbValue = 'My Text'
     
 
+class TestMarkdownText(MetaAttrTestCase):
+
+    fieldClass = dm.dom.meta.MarkdownText
+    typeName = 'MarkdownText'
+    isValueObject = True
+    htmlValue = 'My Text'
+    htmlRepr = 'My Text'
+    domValue = 'My Text'
+    dbValue = 'My Text'
+    
+
 class TestUrl(MetaAttrTestCase):
 
     fieldClass = dm.dom.meta.Url
@@ -151,13 +165,27 @@ class TestDate(MetaAttrTestCase):
         self.failUnless(self.field.convertor)
 
 
+class TestRDate(MetaAttrTestCase):
+    
+    fieldClass = dm.dom.meta.RDate
+    typeName = 'RDate'
+    isValueObject = True
+    htmlValue = '03-06-2007'
+    htmlRepr = '03-06-2007'
+    domValue = mx.DateTime.Date(2007, 6, 3)
+    dbValue = mx.DateTime.Date(2007, 6, 3)
+    
+    def test_convertor(self):
+        self.failUnless(self.field.convertor)
+
+
 class TestBoolean_True(MetaAttrTestCase):
     
     fieldClass = dm.dom.meta.Boolean
     typeName = 'Boolean'
     isValueObject = True
     htmlValue = 'True'
-    htmlRepr = True
+    htmlRepr = 'on'
     domValue = True
     dbValue = True
     
@@ -168,7 +196,7 @@ class TestBoolean_False(MetaAttrTestCase):
     typeName = 'Boolean'
     isValueObject = True
     htmlValue = 'False'
-    htmlRepr = False
+    htmlRepr = ''
     domValue = False
     dbValue = False
 
@@ -179,7 +207,7 @@ class TestInteger_Zero(MetaAttrTestCase):
     typeName = 'Integer'
     isValueObject = True
     htmlValue = '0'
-    htmlRepr = '0'
+    htmlRepr = 0
     domValue = 0
     dbValue = 0
 
@@ -190,7 +218,7 @@ class TestInteger_NonZero(MetaAttrTestCase):
     typeName = 'Integer'
     isValueObject = True
     htmlValue = '12'
-    htmlRepr = '12'
+    htmlRepr = 12
     domValue = 12
     dbValue = 12
 
@@ -201,7 +229,7 @@ class TestInteger_Negative(MetaAttrTestCase):
     typeName = 'Integer'
     isValueObject = True
     htmlValue = '-12'
-    htmlRepr = '-12'
+    htmlRepr = -12
     domValue = -12
     dbValue = -12
 
