@@ -15,8 +15,9 @@ from dm.webkit import Context
 from dm.webkit import template_loader
 import dm.times
 import os
+import mx.DateTime
 
-moddebug = True
+moddebug = False
 
 class ControlledAccessView(object):
 
@@ -28,14 +29,15 @@ class ControlledAccessView(object):
     registry         = RequiredFeature('DomainRegistry')
     debug            = RequiredFeature('Debug')  # deprecated
     isDebug          = RequiredFeature('Debug')  # rename for debug
+    systemMode       = RequiredFeature('SystemMode')
 
     authSessionCookieName = dictionary[AUTH_COOKIE_NAME]
     noAuthSessionCookieString = dictionary[NO_AUTH_COOKIE_NAME]
     if AUTH_COOKIE_DOMAIN in dictionary:
         authCookieDomain = dictionary[AUTH_COOKIE_DOMAIN]
     else:
-        # Prefix with '.' to match subdomains.
-        authCookieDomain = '.' + dictionary[DOMAIN_NAME]
+        # Default to just matching requested domain.
+        authCookieDomain = None  
 
     def __init__(self, request=None, masterView=None, **kwds):
         self.request = request
@@ -47,7 +49,8 @@ class ControlledAccessView(object):
         self.redirect = ''
         self.redirectPath = ''
         self.returnPath = ''
-        if self.isSlaveView(): self.initFromMaster()
+        if self.isSlaveView():
+            self.initFromMaster()
 
         # Todo: Clean up all this guff.
         self._canReadSystem = None
@@ -87,17 +90,22 @@ class ControlledAccessView(object):
         
     def setSessionFromCookieString(self, cookieString):
         if not cookieString:
-            if self.debug:
-                msg = 'No authentication session cookie in request.'
-                self.logger.debug(msg)
+            msg = '%s: No authentication session cookie in request.' % self.__class__.__name__
+            self.logger.error(msg)
             return
+        if moddebug:
+            self.logger.info(
+                '%s: Resuming session from cookie %s...' % (
+                    self.__class__.__name__, cookieString[0:10] 
+                )
+            )
         if cookieString == self.noAuthSessionCookieString:
-            if self.debug:
-                msg = 'Unauthenticated session cookie in request.'
+            if moddebug:
+                msg = '%s: Unauthenticated session cookie in request.' % self.__class__.__name__
                 self.logger.debug(msg)
             return
-        if self.debug:
-            msg = 'Session cookie in request... (%s)' % (
+        if moddebug:
+            msg = '%s: Session cookie in request.' % (
                 self.__class__.__name__
             )
             self.logger.debug(msg)
@@ -106,27 +114,34 @@ class ControlledAccessView(object):
         except KforgeSessionCookieValueError:
             self.session = None
             self.logger.error(
-                'Session cookie value error: %s' % cookieString
+                '%s: Session cookie value error: %s' % (self.__class__.__name__, cookieString)
             )
         else:
-            self.session = self.findSession(sessionKey)
-            if self.session:
-                self.session.updateLastVisited()
-                if self.debug:
-                    self.logger.debug(
-                        'Session re-established (%s): %s' % (
-                            self.__class__.__name__, self.session
+            try:
+                self.session = self.findSession(sessionKey)
+                if self.session:
+                    self.session.updateLastVisited()
+                    if moddebug:
+                        self.logger.debug(
+                            '%s: Session %s resumed from cookie.' % (
+                                self.__class__.__name__, sessionKey
+                            )
                         )
+                else:
+                    self.logger.error(
+                        '%s: No session for key in cookie: %s' % (self.__class__.__name__, sessionKey)
                     )
-            else:
-                if self.debug:
-                    self.logger.debug(
-                        'No session for sessionKey: %s' % sessionKey
-                    )
+                    self.isSessionStopping = True
+            except Exception, inst:
+                self.logger.error(
+                    '%s: Error finding session: %s' % (self.__class__.__name__, cookieString)
+                )
+                self.isSessionStopping = True
 
     def makeSessionKeyFromCookieString(self, cookieString):
         validateCookieStrategy = ValidateCookieString(cookieString)
-        return validateCookieStrategy.validate()
+        plainString = validateCookieStrategy.validate()
+        return plainString
 
     def makeCookieStringFromSessionKey(self, sessionKey):
         cookieStringStrategy = MakeCookieString(sessionKey)
@@ -139,11 +154,9 @@ class ControlledAccessView(object):
         return checkString
 
     def findSession(self, sessionKey):
-        if not sessionKey:
-            return None
-        if sessionKey in self.registry.sessions:
+        try:
             return self.registry.sessions[sessionKey]
-        else:
+        except KforgeRegistryKeyError:
             return None
 
     def checkAccessControl(self):
@@ -393,6 +406,7 @@ class SessionView(ControlledAccessView):
         self.project = None
         self.member = None
         self.service = None
+        self.handlingStarted = mx.DateTime.now()
         self.readRequest()
 
     def __repr__(self):
@@ -402,17 +416,18 @@ class SessionView(ControlledAccessView):
 
     def readRequest(self):
         if not self.request:
-            if self.debug:
-                self.logger.debug('No request object in view.')
+            if moddebug:
+                self.logger.info('%s: No request object.' % self.__class__.__name__)
             return
         self.setPathFromRequest()
-        self.readRequestSession()
-        self.logger.info('Request to %s: %s' % (
-            self.__class__.__name__, self.path
+        self.logger.info('%s: Handling %s request to path %s' % (
+            self.__class__.__name__, self.getMethodName(), self.path
         ))
+        self.readRequestSession()
 
     def setPathFromRequest(self):
-        if self.path != None: return
+        if self.path != None:
+            return
         uriPrefix = self.dictionary[URI_PREFIX]
         requestPath = self.request.path 
         if uriPrefix and requestPath.startswith(uriPrefix):
@@ -426,13 +441,10 @@ class SessionView(ControlledAccessView):
             return
         cookieName = self.authSessionCookieName
         cookieString = self.request.COOKIES.get(cookieName, '')
-        self.setSessionFromCookieString(cookieString)
-        if self.session:
-            self.session.updateLastVisited()
+        if cookieString:
+            self.setSessionFromCookieString(cookieString)
         else:
-            if moddebug and self.debug:
-                msg = 'No session for cookieString: %s' % cookieString
-                self.logger.debug(msg)
+            pass
         
     def getRequestParam(self, paramName, default=None):
         params = self.getRequestParams()
@@ -441,8 +453,8 @@ class SessionView(ControlledAccessView):
 
     def getRequestParams(self):
         if self.requestParams == None:
-            if moddebug and self.debug:
-                msg = "Reading request params from request..."
+            if moddebug:
+                msg = "%s: Reading request params from request." % (self.__class__.__name__)
                 self.logger.debug(msg)
             if self.request.POST:
                 self.requestParams = self.request.POST.copy()
@@ -452,9 +464,9 @@ class SessionView(ControlledAccessView):
                 self.requestParams = self.getZeroRequestParams()
             #if self.request.FILES:
             #    self.requestParams.update(self.request.FILES)
-        if moddebug and self.debug:
-            msg = "Request params: %s" % self.requestParams
-            self.logger.debug(msg)
+            if moddebug:
+                msg = "%s: Request params: %s" % (self.__class__.__name__, self.requestParams)
+                self.logger.debug(msg)
         return self.requestParams
 
     def getZeroRequestParams(self):
@@ -463,10 +475,7 @@ class SessionView(ControlledAccessView):
 
     def getResponse(self):
         if self.checkAccessControl():
-            self.logViewResponse(self.getMethodName(), 'OK')
             self.takeAction()
-        else:
-            self.logViewResponse(self.getMethodName(), 'DENIED')
         if not self.redirect:
             self.createContext()
         if not self.redirect:
@@ -477,6 +486,9 @@ class SessionView(ControlledAccessView):
                 self.setContext()
         self.createResponse()
         self.setSessionCookie()
+        if moddebug:
+            msg = "%s: Returning response: %s" % (self.__class__.__name__, repr(self.response))
+            self.logger.info(msg)
         return self.response
 
     def logViewResponse(self, methodName, accessDecisionName):
@@ -485,11 +497,11 @@ class SessionView(ControlledAccessView):
         else:
             userName = 'visitor'
         self.logger.info(
-            "%s %s %s %s %s" % (
+            "%s: Response summary: %s %s %s %s" % (
+                self.__class__.__name__,
                 accessDecisionName,
                 userName,
                 methodName,
-                self.__class__.__name__,
                 self.path,
             )
         )
@@ -573,9 +585,9 @@ class SessionView(ControlledAccessView):
     def renderContextWithTemplate(self):
         template = self.getTemplate()
         context = self.getContext()
-        if moddebug and self.debug:
+        if moddebug:
             self.logger.debug("Rendering template '%s' for context: %s" % (
-                self.templatePath, context
+                self.templatePath, "context printing suspended" #context
             ))
         return template.render(context)
 
@@ -591,7 +603,7 @@ class SessionView(ControlledAccessView):
             raise Exception(msg)
         templatePath = self.getTemplatePath()
         templatePath += ".html"
-        if moddebug and self.debug:
+        if moddebug:
             self.logger.debug("Loading template: %s" % templatePath)
         try:
             self.loadTemplateFromPath(templatePath)
@@ -678,7 +690,7 @@ class SessionView(ControlledAccessView):
     def setRedirect(self, redirectPath):
         if redirectPath:
             redirectPath = self.dictionary[URI_PREFIX] + redirectPath
-            self.logger.info('Redirecting to: %s' % redirectPath)
+            self.logger.info('%s: Redirecting to: %s' % (self.__class__.__name__, redirectPath))
             self.redirect = redirectPath
     
     def setRedirectLogin(self, returnPath=''):
@@ -686,7 +698,7 @@ class SessionView(ControlledAccessView):
             if self.request:
                 returnPath = self.path
         if returnPath:
-            self.logger.info('Returning later to: %s' % returnPath)
+            self.logger.debug('%s: Returning later to: %s' % (self.__class__.__name__, returnPath))
         self.setRedirect('/login/' + returnPath)
     
     def setRedirectAccessDenied(self):
@@ -697,42 +709,60 @@ class SessionView(ControlledAccessView):
             self.setRedirectLogin()
     
     def setSessionCookie(self):
-        "Sets a cookie identifying the session in the response."
-        cookieString = ''
-        if self.session:
-            cookieString = self.makeCookieStringFromSessionKey(self.session.key)
-        else:
-            cookieString = self.noAuthSessionCookieString
+        "Sets (or deletes) a cookie identifying the session in the response."
         cookieName = self.authSessionCookieName
         cookieDomain = self.authCookieDomain
-        if self.getRequestParam('rememberme'):
-            maxAge = 315360000
-        else:
-            maxAge = None
-        if self.debug:
-            msg = 'Setting cookie %s: %s' % (cookieName, cookieString)
-            self.logger.debug(msg)
-        webkitName = self.dictionary[WEBKIT_NAME]
-        if webkitName == 'django':
+        cookiePath = self.dictionary[URI_PREFIX].strip() or '/'
+        if hasattr(self, 'isSessionStopping'):
+            if moddebug:
+                msg = "%s: Deleting '%s' cookie." % (
+                    self.__class__.__name__, cookieName
+                )
+                self.logger.info(msg)
+            webkitName = self.dictionary[WEBKIT_NAME]
+            self.response.delete_cookie(
+                cookieName, domain=cookieDomain, path=cookiePath
+            )
+        elif self.session and hasattr(self, 'isSessionStarting'):
+            cookieString = self.makeCookieStringFromSessionKey(self.session.key)
+            if self.getRequestParam('rememberme'):
+                maxAge = 315360000
+            else:
+                maxAge = None
+            if moddebug:
+                msg = "%s: Setting '%s' cookie: %s." % (
+                    self.__class__.__name__, cookieName, self.session.key[0:32]
+                )
+                self.logger.info(msg)
+            webkitName = self.dictionary[WEBKIT_NAME]
             self.response.set_cookie(
-                cookieName, cookieString, domain=cookieDomain, max_age=maxAge,
-                expires=maxAge
+                cookieName, cookieString, domain=cookieDomain, 
+                max_age=maxAge, expires=maxAge, path=cookiePath
             )
-        elif webkitName == 'pylons':
-            print "Todo: How to set the pylons cookie? Use AuthKit"
 
     def startSession(self, person):
-        self.session = person.sessions.create()
         self.person = person
-        self.logger.info("Started session: %s" % self.session)
+        self.session = person.sessions.create()
+        self.isSessionStarting = True
+        if moddebug:
+            msg = "%s: Started session: %s" % (
+                self.__class__.__name__, self.session.key
+            )
+            self.logger.info(msg)
         
     def stopSession(self):
         if self.session:
-            self.logger.info("Stopping session: %s" % self.session)
-            self.session.delete()
+            oldKey = self.session.key
+            if moddebug:
+                self.logger.info("%s: Stopping session: %s" % (self.__class__.__name__, oldKey))
+            self.isSessionStopping = True
+            try:
+                self.session.delete()
+            except Exception, inst:
+                self.logger.error("%s: Error deleting session %s: %s" % (self.__class__.__name__, oldKey, repr(inst)))
             self.session = None
         else:
-            self.logger.info("Stopping session, but there was no session.")
+            self.logger.warn("%s: Stopping session, but there was no session." % (self.__class__.__name__))
 
     def showInlineHelp(self):
         return False
@@ -740,6 +770,20 @@ class SessionView(ControlledAccessView):
     def hasListallPage(self):
         return False
 
+    def showSystemStatusMsg(self):
+        return not self.systemMode.isProduction()
+
+    def getSystemStatusMsg(self):
+        msg = "%s" % self.dictionary.get(SYSTEM_MODE)
+        msg +=" %s " % self.dictionary[LOG_LEVEL]
+        import dm
+        import dm.db
+        import dm.webkit
+        msg +=" domainmodel==%s" % (dm.__version__)
+        msg +=" %s==%s" % (dm.webkit.webkitName, dm.webkit.webkitVersionFull or dm.webkit.webkitVersion)
+        msg +=" sqlobject==%s" % (dm.db.sqoVer)
+        msg +=" (%ss)" % str(float(mx.DateTime.now() - self.handlingStarted))[0:5]
+        return msg
 
 class AbstractClassView(SessionView):
 
@@ -803,21 +847,16 @@ class AbstractInstanceView(AbstractClassView):
 
     def buildManipulator(self):
         manipulatorClass = self.getManipulatorClass()
-        if not issubclass(manipulatorClass, DomainObjectManipulator):
-            msg = "Manipulator class isn't a DomainObjectManipulator: %s" % (
-                manipulatorClass
-            )
-            raise Exception(msg)
         objectRegister = self.getManipulatedObjectRegister()
         domainObject = self.getManipulatedDomainObject()
         fieldNames = self.getManipulatedFieldNames()
-        if moddebug and self.debug:
-            msg = "Building manipulator %s for position %s" % (
+        if moddebug:
+            msg = "%s: Building manipulator %s for position %s" % (
+                self.__class__.__name__,
                 manipulatorClass.__name__,
                 self.getViewPosition()
             )
-            self.logger.debug(msg)
-            
+            self.logger.info(msg)
         return manipulatorClass(objectRegister, domainObject, fieldNames, self)
 
     def getManipulatedFieldNames(self):
@@ -840,7 +879,7 @@ class AbstractFormView(AbstractInstanceView):
             if not self.getFormErrors():
                 self.setPostManipulateRedirect()
             else:
-                self.logger.warning("Form errors on %s: %s" % (
+                self.logger.warning("%s: Form errors: %s" % (
                     self.__class__.__name__, self.formErrors
                 ))
         elif not self.getRequestParams():
@@ -861,7 +900,7 @@ class AbstractFormView(AbstractInstanceView):
         if self.formErrors == None:
             if self.getRequestParams():
                 self.formErrors = self.getValidationErrors()
-                if moddebug and self.debug:
+                if moddebug:
                     self.logger.debug("Form errors: %s" % self.formErrors)
             else:
                 self.formErrors = self.getZeroFormErrors() 
@@ -874,7 +913,7 @@ class AbstractFormView(AbstractInstanceView):
         manipulator = self.getManipulator()
         requestParams = self.getRequestParams()
         validationErrors = manipulator.getValidationErrors(requestParams)
-        if moddebug and self.debug:
+        if moddebug:
             self.logger.debug("Validation errors: %s" % validationErrors)
         return validationErrors
             
@@ -953,6 +992,14 @@ class AbstractListView(AbstractClassView):
             'showRegisterSearch': False,
         })
 
+    def present(self):
+        register = self.getManipulatedObjectRegister()
+        if dm.webkit.webkitName == 'django' and dm.webkit.webkitVersion == '1.0':
+            list = [o for o in register]
+        elif dm.webkit.webkitName == 'django' and dm.webkit.webkitVersion == '0.96':
+            list = [o.present() for o in register]
+        return list
+
 
 class AbstractPendingView(AbstractListView):
 
@@ -989,7 +1036,7 @@ class AbstractSearchView(AbstractClassView):
                 'objectCount': objectCount,
                 'isCountZero': objectCount == 0,
                 'isCountSingle': objectCount == 1,
-                'userQuery': self.userQuery.replace('"', '&quot;'),
+                'userQuery': self.userQuery,
                 'startsWith': self.startsWith,
                 'showRegisterTable': bool(objectCount),
                 # todo: Improve above HTML character substitution.
@@ -1078,7 +1125,6 @@ class AbstractCreateView(AbstractInputFormView):
         objectRegister = self.getManipulatedObjectRegister()
         domainClass = self.getDomainClass(objectRegister.typeName)
         for metaAttr in domainClass.meta.attributes:
-            #if not (hasattr(metaAttr, 'default') and metaAttr.default):
             if not (hasattr(metaAttr, 'default')):
                 continue
             if metaAttr.isAssociateList:
@@ -1096,9 +1142,10 @@ class AbstractCreateView(AbstractInputFormView):
             manipulator = self.getManipulator()
             requestParams = self.getCreateParams()
             manipulator.create(requestParams)
-            if not manipulator.domainObject:
-                raise Exception("Create manipulator did not produce an object.")
-            self.setCreatedObject(manipulator.domainObject)
+            if not manipulator.manipulationErrors:
+                self.setCreatedObject(manipulator.domainObject)
+            else:
+                self.formErrors = manipulator.manipulationErrors 
 
     def getCreateParams(self):
         createParams = self.getRequestParams()
@@ -1199,6 +1246,8 @@ class AbstractUpdateView(AbstractInputFormView):
             manipulator = self.getManipulator()
             requestParams = self.getUpdateParams()
             manipulator.update(requestParams)
+            if manipulator.manipulationErrors:
+                self.formErrors = manipulator.manipulationErrors 
         else:
             if self.debug:
                 self.logger.debug('Manipulator update not possible with form errors: %s' % (formErrors))
