@@ -5,9 +5,11 @@ from dm.webkit import Manipulator
 from dm.webkit import htmlescape
 from dm.webkit import webkitName, webkitVersion
 if webkitName == 'django' and webkitVersion == '1.0':
-    from dm.webkit import SortedDict
+    from dm.webkit import SortedDict, MultipleChoiceField
 import re
 import dm.webkit
+from dm.dictionarywords import DB_MIGRATION_IN_PROGRESS
+import traceback
 
 class BaseManipulator(Manipulator):
     "Supertype for domain model manipulators."
@@ -21,26 +23,29 @@ class BaseManipulator(Manipulator):
     sizeSelectMultiple = 4
 
     base_fields = []  # Django 1.0.
+    
+    manipulationErrors = {}
 
     def getValidationErrors(self, data):
-            if webkitName == 'django' and webkitVersion == '1.0':
-                self._errors = None
-                self.data = data
-                self.is_bound = True
-                return self.errors
-            else:
-                try:
-                    return self.get_validation_errors(data)
-                except Exception, inst:
-                    msg = "Exception in 'getValidationErrors' with data %s: %s" % (
-                        data, inst
-                    )
-                    self.logger.error(msg)
-                    raise
+        if webkitName == 'django' and webkitVersion == '1.0':
+            self._errors = None
+            self.data = data
+            self.is_bound = True
+            return self.errors
+        else:
+            try:
+                return self.get_validation_errors(data)
+            except Exception, inst:
+                msg = "Error in 'getValidationErrors' with data %s: %s" % (
+                    data, inst
+                )
+                self.logger.error(msg)
+                raise
 
     def decodeHtml(self, data):
         if webkitName == 'django' and webkitVersion == '0.96':
             return self.do_html2python(data)
+        return data
 
     def isTwoCharsMin(self, field_data, all_data):
         self.isTooShort(field_data, 2)
@@ -80,6 +85,10 @@ class DomainObjectManipulator(BaseManipulator):
         self.objectRegister = objectRegister
         self.metaObject = self.objectRegister.getDomainClassMeta()
         self.domainObject = domainObject
+        msg = "Building %s for %s." % (
+            self.__class__.__name__, self.metaObject.name
+        )
+        self.logger.debug(msg)
         self.fieldNames = fieldNames
         if self.fieldNames == None:
             self.fieldNames = []
@@ -87,38 +96,39 @@ class DomainObjectManipulator(BaseManipulator):
             self.fields = SortedDict()
         else:
             self.fields = []
-        msg = "Building manipulator fields for %s..." % (
-            self.__class__.__name__
-        )
-        self.logger.debug(msg)
         self.buildFields()
         if self.fieldNames == []:
             if webkitName == 'django' and webkitVersion == '1.0':
                 self.fieldNames = self.fields.keys()
             else:
                 self.fieldNames = [field.field_name for field in self.fields]
-        msg = "Built '%s' manipulator with field names: %s" % (
+        msg = "Built %s manipulator with fields: %s" % (
             self.__class__.__name__, ", ".join(self.fieldNames)
         )
         self.logger.debug(msg)
 
     def buildFields(self):
         if self.fieldNames:
-            msg = "Building manipulator field objects from fieldNames"
-            msg = "%s (%s)." % (msg, ", ".join(self.fieldNames))
+            msg = "Building fields from fieldNames..."
+            msg += "%s." % ", ".join(self.fieldNames)
             self.logger.debug(msg)
             for fieldName in self.fieldNames:
                 metaAttr = self.metaObject.attributeNames[fieldName]
                 self.buildField(metaAttr)
         else:
             self.fieldNames = []
-            msg = "Manipulator building fields from domain object meta..."
+            msg = "Building fields from meta object..."
             self.logger.debug(msg)
             for metaAttr in self.metaObject.attributes:
-                if not self.isAttrExcluded(metaAttr):
+                if self.isAttrExcluded(metaAttr):
+                    msg = "Excluded '%s' attribute." % metaAttr.name
+                    self.logger.debug(msg)
+                else:
                     self.buildField(metaAttr)
 
     def isAttrExcluded(self, metaAttr):
+        if self.dictionary[DB_MIGRATION_IN_PROGRESS]:
+            return False
         if self.isAttrNameExcluded(metaAttr.name):
             return True
         if not metaAttr.isEditable:
@@ -134,12 +144,12 @@ class DomainObjectManipulator(BaseManipulator):
         field = None
         isFieldRequired = metaAttr.isRequired
         self.logger.debug("Building manipulator field: %s" % metaAttr.name)
-
         if metaAttr.isAssociateList:
             countChoices = metaAttr.countChoices(self.domainObject)
             choices = metaAttr.getAllChoices(self.domainObject)
             if webkitName == 'django' and webkitVersion == '1.0':
-                field = self.webkit.ChoiceField(
+                # Todo: Write test to check for this (was: ChoiceField).
+                field = self.webkit.MultipleChoiceField(
                     required=isFieldRequired,
                     choices=choices,
                 )
@@ -161,7 +171,7 @@ class DomainObjectManipulator(BaseManipulator):
             # instead of drop-down when more than count limit.
             if True or countChoices <= 50:
                 choices = metaAttr.getAllChoices(self.domainObject)
-                choices = [('', '-- select option --')] + choices
+                choices = [(u'', u'-- select option --')] + choices
                 if webkitName == 'django' and webkitVersion == '1.0':
                     field = self.webkit.ChoiceField(
                         required=isFieldRequired,
@@ -179,7 +189,10 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             else:  # need to find a way to only do this for string keys
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.CharField(
+                        required=isFieldRequired,
+                        choices=choices,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.TextField(
                         field_name=metaAttr.name,
@@ -226,7 +239,10 @@ class DomainObjectManipulator(BaseManipulator):
                 
             elif metaAttr.typeName == 'Text':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.CharField(
+                        required=isFieldRequired,
+                        widget=self.webkit.widgets.Textarea(),
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.LargeTextField(
                         field_name=metaAttr.name,
@@ -241,7 +257,10 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             elif metaAttr.typeName == 'MarkdownText':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.CharField(
+                        required=isFieldRequired,
+                        widget=self.webkit.widgets.Textarea(),
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.LargeTextField(
                         field_name=metaAttr.name,
@@ -272,7 +291,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             elif metaAttr.typeName == 'Integer':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.IntegerField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.IntegerField(
                         field_name=metaAttr.name,
@@ -285,7 +306,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             elif metaAttr.typeName == 'Boolean':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.BooleanField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.CheckboxField(
                         field_name=metaAttr.name
@@ -298,7 +321,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             elif metaAttr.typeName == 'Url':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.URLField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.URLField(
                         field_name=metaAttr.name,
@@ -326,7 +351,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             elif metaAttr.typeName == 'Date':
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.DateField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.DateField(
                         field_name=metaAttr.name,
@@ -337,9 +364,11 @@ class DomainObjectManipulator(BaseManipulator):
                         metaAttr.name,
                         size=40
                     )
-            elif metaAttr.typeName == 'RDate':
+            elif metaAttr.typeName == 'RDate': # Todo: Remove this type.
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.DateField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.RDateField(
                         field_name=metaAttr.name,
@@ -352,7 +381,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
             else:
                 if webkitName == 'django' and webkitVersion == '1.0':
-                    raise Exception("No field yet.")
+                    field = self.webkit.CharField(
+                        required=isFieldRequired,
+                    )
                 elif webkitName == 'django' and webkitVersion == '0.96':
                     field = self.webkit.TextField(
                         field_name=metaAttr.name,
@@ -365,7 +396,9 @@ class DomainObjectManipulator(BaseManipulator):
                     )
         elif metaAttr.typeName == 'ImageFile':
             if webkitName == 'django' and webkitVersion == '1.0':
-                raise Exception("No field yet.")
+                field = self.webkit.ImageField(
+                    required=isFieldRequired,
+                )
             elif webkitName == 'django' and webkitVersion == '0.96':
                 field = self.webkit.FileUploadField(
                     field_name=metaAttr.name,
@@ -377,35 +410,51 @@ class DomainObjectManipulator(BaseManipulator):
                         size=40
                     )
         if field:
+            field.metaAttr = metaAttr
+            field.field_title = metaAttr.calcTitle()
+            field.field_comment = metaAttr.comment
             if webkitName == 'django' and webkitVersion == '1.0':
-                field.metaAttr = metaAttr
                 field.field_name = metaAttr.name
-                field.field_title = metaAttr.calcTitle()
-                field.field_comment = metaAttr.comment
                 self.fields[metaAttr.name] = field
             elif webkitName == 'django' and webkitVersion == '0.96':
-                field.metaAttr = metaAttr
-                field.field_title = metaAttr.calcTitle()
-                field.field_comment = metaAttr.comment
-                self.fields.append(field)
-            elif webkitName == 'pylons':
                 self.fields.append(field)
         else:
-            message = "No form field for meta attribute: %s" % metaAttr
+            message = "Can't build manipulator field for meta attribute: %s" % metaAttr
             self.logger.critical(message)
             raise Exception, message
 
     def create(self, data):
-        # todo: rename 'data'
-        self.data = data
-        self.createObjectKwds()
-        self.createDomainObject()
-        self.setAssociateListAttributes()
+        if webkitName == 'django' and webkitVersion == '1.0':
+            self.cleaned_data = data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            self.data = data
+        try:
+            self.createObjectKwds()
+            self.createDomainObject()
+            try:
+                self.setAssociateListAttributes()
+            except:
+                trace = traceback.format_exc()
+                self.logger.error("Model: Error raising create event: %s" % trace)
+                try:
+                    self.domainObject.uncreate()
+                except Exception, inst:
+                    trace = traceback.format_exc()
+                    self.logger.error("Model: Couldn't undo create: %s" % trace)
+                raise
+        except Exception, inst:
+            msg = "System error creating object for data: %s: %s" % (data, repr(inst))
+            self.logger.error(msg)
+            self.manipulationErrors = {'': ["System error: %s" % str(inst)]}
         
     def createObjectKwds(self):
         self.objectKwds = {}
         fieldedAttrs = self.getFieldedMetaAttrs()
-        domValues = self.presentationToDomainValues(self.data, fieldedAttrs)
+        if webkitName == 'django' and webkitVersion == '1.0':
+            data = self.cleaned_data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            data = self.data
+        domValues = self.presentationToDomainValues(data, fieldedAttrs)
         for (attrName, domValue) in domValues.items():
             if not domValue == None:
                 self.objectKwds[attrName] = domValue
@@ -441,10 +490,14 @@ class DomainObjectManipulator(BaseManipulator):
 
     def getFieldedMetaAttrs(self):
         metaAttrs = {}
+        if webkitName == 'django' and webkitVersion == '1.0':
+            data = self.cleaned_data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            data = self.data
         for metaAttr in self.metaObject.attributes:
             if not metaAttr.isAssociateList:
                 if metaAttr.name in self.fieldNames:
-                    if metaAttr.name in self.data:
+                    if metaAttr.name in data:
                         metaAttrs[metaAttr.name] = metaAttr
         return metaAttrs
 
@@ -464,12 +517,24 @@ class DomainObjectManipulator(BaseManipulator):
         return domValues
 
     def update(self, data):
-        self.data = data
-        self.setNonAssociateListAttributes()
-        self.setAssociateListAttributes()
+        if webkitName == 'django' and webkitVersion == '1.0':
+            self.cleaned_data = data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            self.data = data
+        try:
+            self.setNonAssociateListAttributes()
+            self.setAssociateListAttributes()
+        except Exception, inst:
+            msg = "System error updating object for data: %s: %s" % (data, repr(inst))
+            self.logger.error(msg)
+            self.manipulationErrors = {'': ["System error: %s" % str(inst)]}
 
     def setNonAssociateListAttributes(self):
-        self.setAttributesFromMultiValueDict(self.domainObject, self.data)
+        if webkitName == 'django' and webkitVersion == '1.0':
+            data = self.cleaned_data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            data = self.data
+        self.setAttributesFromMultiValueDict(self.domainObject, data)
         self.domainObject.save()
        
     def setAttributesFromMultiValueDict(self, domainObject, multiValueDict):
@@ -489,10 +554,15 @@ class DomainObjectManipulator(BaseManipulator):
                 self.setAssociateListAttribute(metaAttr)
            
     def setAssociateListAttribute(self, metaAttr):
+        # Todo: Test for setting (and clearing!) lists of associates.
+        if webkitName == 'django' and webkitVersion == '1.0':
+            data = self.cleaned_data
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            data = self.data
         if metaAttr.name in self.fieldNames:
-            if self.data.has_key(metaAttr.name):
+            if data.has_key(metaAttr.name):
                 metaAttr.setAttributeFromMultiValueDict(
-                    self.domainObject, self.data
+                    self.domainObject, data
                 )
 
     def getAttributeField(self, attrName):
@@ -502,10 +572,13 @@ class DomainObjectManipulator(BaseManipulator):
         return None
 
     def getUpdateParams(self):
-        initialParams = self.domainObject.asRequestParams()
         if webkitName == 'django' and webkitVersion == '1.0':
-            self.initial = initialParams
-        return initialParams
+            # Form fields are initialized with these data. 
+            self.initial = self.domainObject.asDictValues()
+            # Just in case. :-)
+            return self.initial #domainObject.asRequestParams()
+        elif webkitName == 'django' and webkitVersion == '0.96':
+            return self.domainObject.asRequestParams()
 
 
 class HasManyManipulator(DomainObjectManipulator):
@@ -549,6 +622,16 @@ class FormWrapper(object):
     def __repr__(self):
         return repr(self.data)
 
+    def getErrorMessages(self):
+        errorMessages = []
+        for field in self.fields:
+            if field.error:
+                errorMessages.append(field.error)
+        for error in self.error_dict.get('', []):
+            if error:
+                errorMessages.append(error)
+        return errorMessages
+
     def wrapManipulatorFields(self):
         wrappedFields = []
         if webkitName == 'django' and webkitVersion == '1.0':
@@ -614,25 +697,28 @@ class FormWrapper(object):
         raise KeyError
 
     def has_errors(self):
+        return self.hasErrors()
+
+    def hasErrors(self):
         return self.error_dict != {}
 
-
+# Todo: Test for incorrect setting of multiplechoicefield data (only one value was being selected).
 class FormFieldWrapper(object):
     "A bridge between the template system and an individual form field. Used by FormWrapper."
     
     def __init__(self, formfield, data, error_list):
+        #if formfield.field_name == 'svn':
+        #    raise Exception, repr(data)
         self.formfield, self.data, self.error_list = formfield, data, error_list
         self.field_name = self.formfield.field_name
         if webkitName == 'django' and webkitVersion == '1.0':
-            if hasattr(self.formfield.field, 'field_title'):
-                self.field_title = self.formfield.field.field_title
-            if hasattr(self.formfield.field, 'field_comment'):
-                self.field_comment = self.formfield.field.field_comment
+            field = self.formfield.field
         elif webkitName == 'django' and webkitVersion == '0.96':
-            if hasattr(self.formfield, 'field_title'):
-                self.field_title = self.formfield.field_title
-            if hasattr(self.formfield, 'field_comment'):
-                self.field_comment = self.formfield.field_comment
+            field = self.formfield
+        if hasattr(field, 'field_title'):
+            self.field_title = field.field_title
+        if hasattr(field, 'field_comment'):
+            self.field_comment = field.field_comment
         self.error = self.makeErrorMessage()
 
     def makeErrorMessage(self):
@@ -676,7 +762,19 @@ class FormFieldWrapper(object):
         if webkitName == 'django' and webkitVersion == '1.0':
             return self.__unicode__()
         elif webkitName == 'django' and webkitVersion == '0.96':
-            html = str(self.formfield.render(self.data))
+            try:
+                html = self.formfield.render(self.data)
+                if type(html) == unicode:
+                    html = html.encode('utf-8')
+            except UnicodeDecodeError, inst:
+                msg = "(UnicodeDecodeError) Perhaps string with encoded unicode has been added to actual unicode string somewhere along the line. The formfield, data, and any choices: %s %s %s. The error was: %s" % (repr(self.formfield), repr(self.data), getattr(self.formfield, 'choices', ''), repr(inst))
+                raise Exception, msg                
+            except UnicodeEncodeError, inst:
+                msg = "(UnicodeEncodeError) Perhaps string with encoded unicode has been added to actual unicode string somewhere along the line. The formfield, data, and any choices: %s %s %s. The error was: %s" % (repr(self.formfield), repr(self.data), getattr(self.formfield, 'choices', ''), repr(inst))
+                raise Exception, msg                
+            except Exception, inst:
+                msg = "(Exception) Perhaps string with encoded unicode has been added to actual unicode string somewhere along the line. The formfield, data, and any choices: %s %s %s. The error was: %s" % (repr(self.formfield), repr(self.data), getattr(self.formfield, 'choices', ''), repr(inst))
+                raise Exception, msg                
             if self.error_list:
                 return '<div class="field-with-error">' + html + '</div>'
             else:
