@@ -8,11 +8,43 @@ from dm.view.base import AbstractUpdateView
 from dm.view.base import AbstractDeleteView
 from dm.view.manipulator import HasManyManipulator
 
+class RegistryNavigation(object):
+
+    def __init__(self, view):
+        self.view = view
+
+    def createMajorItem(self):
+        return self.createMinorItem()
+
+    def createMinorItem(self):
+        return ''
+
+    def createMinorItems(self):
+        items = []
+        return items
+
+
+class RegistryFieldNames(list):
+
+    def __init__(self, *args, **kwds):
+        super(RegistryFieldNames, self).__init__(*args, **kwds)
+
+
+class RegistryContextSetter(object):
+
+    def __init__(self, *args, **kwds):
+        super(RegistryContextSetter, self).__init__(*args, **kwds)
+
+    def setContext(self, view):
+        pass
+
+
 class RegistryView(AbstractInstanceView):
 
     domainClassName = ''
     manipulatedFieldNames = {}
     manipulators = {}
+    contextSetters = {}
     redirectors = {}
     navigation = {}
     registryPathNames = None
@@ -124,7 +156,8 @@ class RegistryView(AbstractInstanceView):
         for name in registerPathNames:
             if owner:
                 if not name:
-                    raise Exception, "Path names error: %s" % str(registerPathNames)
+                    msg = "Path names error: %s" % str(registerPathNames)
+                    raise Exception, msg
                 register = getattr(owner, name)
                 owner = None
             else:
@@ -161,6 +194,16 @@ class RegistryView(AbstractInstanceView):
             'domainClassName' : self.getManipulatedObjectRegister().typeName,
             'actionName'    : self.actionName,
         })
+        # todo: Write tests for this.
+        contextSetter = self.getContextSetter()
+        if contextSetter != None:
+            contextSetter.setContext(self)
+
+    def getContextSetter(self):
+        viewPosition = self.getViewPosition()
+        if viewPosition in self.contextSetters:
+            return self.contextSetters[viewPosition]
+        return None
 
     def getManipulatorClass(self):
         viewPosition = self.getViewPosition()
@@ -224,11 +267,36 @@ class RegistryView(AbstractInstanceView):
 
 
 class RegistryListView(RegistryView, AbstractListView):
-    pass
+    
+    def __init__(self, actionName='list', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistryListView, self).__init__(**kwds)
 
 
+class RegistryListallView(RegistryListView):
+    
+    def __init__(self, actionName='list', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistryListallView, self).__init__(**kwds)
+
+    def setContext(self, **kwds):
+        super(RegistryListallView, self).setContext(**kwds)
+        self.context.update({
+            'showRegisterTable': True,
+            'showRegisterIndex': True,
+            'showRegisterAllLink': False,
+        })
+
+    def makeTemplatePath(self):
+        return self.getViewPosition() + 'all'
+        
+
 class RegistrySearchView(RegistryView, AbstractSearchView):
 
+    def __init__(self, actionName='search', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistrySearchView, self).__init__(**kwds)
+
     def searchManipulatedRegister(self):
         register = self.getManipulatedObjectRegister()
         # todo: factor out 'startsWith' and 'userQuery' attributes
@@ -249,7 +317,8 @@ class RegistrySearchView(RegistryView, AbstractSearchView):
 
 class RegistryFindView(RegistryView, AbstractFindView):
 
-    def __init__(self, **kwds):
+    def __init__(self, actionName='find', **kwds):
+        kwds['actionName'] = actionName
         super(RegistryFindView, self).__init__(**kwds)
         if self.actionName:
             self.startsWith = self.actionValue
@@ -263,6 +332,10 @@ class RegistryFindView(RegistryView, AbstractFindView):
 
 class RegistryCreateView(RegistryView, AbstractCreateView):
 
+    def __init__(self, actionName='create', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistryCreateView, self).__init__(**kwds)
+
     def setContext(self):
         super(RegistryCreateView, self).setContext()
 
@@ -277,7 +350,8 @@ class RegistryCreateView(RegistryView, AbstractCreateView):
     
 class RegistryReadView(RegistryListView, AbstractReadView):
 
-    def __init__(self, **kwds):
+    def __init__(self, actionName='read', **kwds):
+        kwds['actionName'] = actionName
         super(RegistryReadView, self).__init__(**kwds)
         self.associationObject = None
         
@@ -304,12 +378,21 @@ class RegistryReadView(RegistryListView, AbstractReadView):
 
 class RegistryUpdateView(RegistryReadView, AbstractUpdateView):
 
+    def __init__(self, actionName='update', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistryUpdateView, self).__init__(**kwds)
+
     def defaultPostManipulateLocation(self):
-        return '/' + self.registryPath + '/'
+        locationPath = '/'+ self.registryPath +'/'
+        return locationPath
 
 
 class RegistryDeleteView(RegistryReadView, AbstractDeleteView):
 
+    def __init__(self, actionName='delete', **kwds):
+        kwds['actionName'] = actionName
+        super(RegistryDeleteView, self).__init__(**kwds)
+
     def defaultPostManipulateLocation(self):
         registryPath = '/'.join(self.getRegistryPathNames()[0:-1])
         return '/' + registryPath + '/'
